@@ -153,6 +153,7 @@ button_fontcolor_clicked (GtkWidget *button, void *data)
   GdkRGBA *color = g_new0 (GdkRGBA, 1);
 
   gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (button), color);
+  g_free (cpuFreq->options->fontcolor);
   cpuFreq->options->fontcolor = gdk_rgba_to_string (color);
   g_free (color);
   cpufreq_update_plugin (TRUE);
@@ -167,7 +168,16 @@ combo_changed (GtkWidget *combo, CpuFreqPluginConfigure *configure)
 
   if (GTK_WIDGET (combo) == configure->combo_cpu)
   {
-    cpuFreq->options->show_cpu = selected;
+    guint num_cpus = cpuFreq->cpus->len;
+    if (selected < num_cpus)
+      cpuFreq->options->show_cpu = selected;
+    else if (selected == num_cpus + 0)
+      cpuFreq->options->show_cpu = CPU_MIN;
+    else if (selected == num_cpus + 1)
+      cpuFreq->options->show_cpu = CPU_AVG;
+    else if (selected == num_cpus + 2)
+      cpuFreq->options->show_cpu = CPU_MAX;
+
     cpufreq_update_plugin (TRUE);
   }
 }
@@ -218,8 +228,6 @@ cpufreq_configure (XfcePanelPlugin *plugin)
     "window-close", _("_Close"), GTK_RESPONSE_OK,
     NULL);
 
-  xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dialog), _("Configure the CPU frequency plugin"));
-
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_window_set_icon_name (GTK_WINDOW (dialog), "xfce4-cpufreq-plugin");
   gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
@@ -354,7 +362,27 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("avg"));
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _("max"));
 
-  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cpuFreq->options->show_cpu);
+retry:
+  switch (cpuFreq->options->show_cpu)
+  {
+  case CPU_MIN:
+    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cpuFreq->cpus->len + 0);
+    break;
+  case CPU_AVG:
+    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cpuFreq->cpus->len + 1);
+    break;
+  case CPU_MAX:
+    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cpuFreq->cpus->len + 2);
+    break;
+  default:
+    if (cpuFreq->options->show_cpu >= 0 && cpuFreq->options->show_cpu < (gint) cpuFreq->cpus->len)
+      gtk_combo_box_set_active (GTK_COMBO_BOX (combo), cpuFreq->options->show_cpu);
+    else
+    {
+      cpuFreq->options->show_cpu = CPU_DEFAULT;
+      goto retry;
+    }
+  }
   g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (combo_changed), configure);
 
   /* check buttons for display widgets in panel */
