@@ -39,11 +39,10 @@
 
 CpuFreqPlugin *cpuFreq;
 
-void
+static void
 cpufreq_label_set_font (void)
 {
   gchar *css = NULL, *css_font = NULL, *css_color = NULL;
-  GtkCssProvider *provider;
   PangoFontDescription *font;
 
   if (G_UNLIKELY (cpuFreq->label == NULL))
@@ -53,7 +52,7 @@ cpufreq_label_set_font (void)
   {
     font = pango_font_description_from_string(cpuFreq->options->fontname);
 
-    css_font = g_strdup_printf("font-family: %s; font-size: %dpx; font-style: %s; font-weight: %s;",
+    css_font = g_strdup_printf("font-family: %s; font-size: %dpt; font-style: %s; font-weight: %s;",
       pango_font_description_get_family (font),
       pango_font_description_get_size (font) / PANGO_SCALE,
       (pango_font_description_get_style (font) == PANGO_STYLE_ITALIC ||
@@ -75,12 +74,24 @@ cpufreq_label_set_font (void)
 
   if (css)
   {
+    GtkCssProvider *provider;
+
+    if (cpuFreq->label_css_provider)
+    {
+      gtk_style_context_remove_provider (
+        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (cpuFreq->label)),
+        GTK_STYLE_PROVIDER (cpuFreq->label_css_provider));
+      cpuFreq->label_css_provider = NULL;
+    }
+
     provider = gtk_css_provider_new ();
 
     gtk_css_provider_load_from_data (provider, css, -1, NULL);
     gtk_style_context_add_provider (
-      GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (cpuFreq->label))),
+      GTK_STYLE_CONTEXT (gtk_widget_get_style_context (cpuFreq->label)),
       GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+    cpuFreq->label_css_provider = provider;
   }
 
   g_free (css);
@@ -90,9 +101,40 @@ cpufreq_label_set_font (void)
 
 
 
+/*
+ * Returns a valid string if all CPUs have the same governor,
+ * otherwise returns NULL.
+ */
+static const gchar*
+cpufreq_cpus_single_governor ()
+{
+  const gchar *governor = NULL;
+
+  for (guint i = 0; i < cpuFreq->cpus->len; i++)
+  {
+    CpuInfo *cpu = g_ptr_array_index (cpuFreq->cpus, i);
+
+    if (!cpu->online)
+      continue;
+
+    if (!cpu->cur_governor || cpu->cur_governor[0] == '\0')
+      return NULL;
+
+    if (!governor)
+      governor = cpu->cur_governor;
+    else if (strcmp (governor, cpu->cur_governor) != 0)
+      return NULL;
+  }
+
+  return governor;
+}
+
+
+
 static CpuInfo *
 cpufreq_cpus_calc_min (void)
 {
+  const gchar *const governor = cpufreq_cpus_single_governor ();
   guint freq = 0;
 
   for (guint i = 0; i < cpuFreq->cpus->len; i++)
@@ -109,7 +151,7 @@ cpufreq_cpus_calc_min (void)
   cpuinfo_free (cpuFreq->cpu_min);
   cpuFreq->cpu_min = g_new0 (CpuInfo, 1);
   cpuFreq->cpu_min->cur_freq = freq;
-  cpuFreq->cpu_min->cur_governor = g_strdup (_("current min"));
+  cpuFreq->cpu_min->cur_governor = g_strdup (governor ? governor : _("current min"));
 
   return cpuFreq->cpu_min;
 }
@@ -119,6 +161,7 @@ cpufreq_cpus_calc_min (void)
 static CpuInfo *
 cpufreq_cpus_calc_avg (void)
 {
+  const gchar *const governor = cpufreq_cpus_single_governor ();
   guint freq = 0, count = 0;
 
   for (guint i = 0; i < cpuFreq->cpus->len; i++)
@@ -138,7 +181,7 @@ cpufreq_cpus_calc_avg (void)
   cpuinfo_free (cpuFreq->cpu_avg);
   cpuFreq->cpu_avg = g_new0 (CpuInfo, 1);
   cpuFreq->cpu_avg->cur_freq = freq;
-  cpuFreq->cpu_avg->cur_governor = g_strdup (_("current avg"));
+  cpuFreq->cpu_avg->cur_governor = g_strdup (governor ? governor : _("current avg"));
 
   return cpuFreq->cpu_avg;
 }
@@ -148,6 +191,7 @@ cpufreq_cpus_calc_avg (void)
 static CpuInfo *
 cpufreq_cpus_calc_max (void)
 {
+  const gchar *const governor = cpufreq_cpus_single_governor ();
   guint freq = 0;
 
   for (guint i = 0; i < cpuFreq->cpus->len; i++)
@@ -164,7 +208,7 @@ cpufreq_cpus_calc_max (void)
   cpuinfo_free (cpuFreq->cpu_max);
   cpuFreq->cpu_max = g_new0 (CpuInfo, 1);
   cpuFreq->cpu_max->cur_freq = freq;
-  cpuFreq->cpu_max->cur_governor = g_strdup (_("current max"));
+  cpuFreq->cpu_max->cur_governor = g_strdup (governor ? governor : _("current max"));
 
   return cpuFreq->cpu_max;
 }
@@ -218,11 +262,10 @@ cpufreq_update_label (CpuInfo *cpu)
         gtk_widget_set_size_request (GTK_WIDGET (cpuFreq->label),
           -1, cpuFreq->label_max_width);
       }
-      else
+      else if (label_size.height > cpuFreq->label_max_width)
       {
-        if (label_size.height > cpuFreq->label_max_width)
-          cpuFreq->layout_changed = TRUE;
         cpuFreq->label_max_width = label_size.height;
+        cpuFreq->layout_changed = TRUE;
       }
     }
     else
@@ -232,10 +275,9 @@ cpufreq_update_label (CpuInfo *cpu)
         gtk_widget_set_size_request (GTK_WIDGET (cpuFreq->label),
           cpuFreq->label_max_width, -1);
       }
-      else
+      else if (label_size.width > cpuFreq->label_max_width)
       {
-        if (label_size.width > cpuFreq->label_max_width)
-          cpuFreq->label_max_width = label_size.width;
+        cpuFreq->label_max_width = label_size.width;
         cpuFreq->layout_changed = TRUE;
       }
     }
@@ -387,14 +429,34 @@ cpufreq_current_cpu (void)
 {
   CpuInfo *cpu = NULL;
 
-  if (cpuFreq->options->show_cpu < cpuFreq->cpus->len)
-    cpu = g_ptr_array_index (cpuFreq->cpus, cpuFreq->options->show_cpu);
-  else if (cpuFreq->options->show_cpu == CPU_MIN)
+  if (G_UNLIKELY (cpuFreq->options->show_cpu >= (gint) cpuFreq->cpus->len))
+  {
+    /* Covered use cases:
+     * - The user upgraded the cpufreq plugin to a newer version
+     * - The user changed the number of CPU cores or SMT (hyper-threading) in the BIOS
+     * - The user moved HDD/SSD to a different machine
+     * - The user downgraded the CPU in the machine
+     */
+    cpuFreq->options->show_cpu = CPU_DEFAULT;
+    cpufreq_write_config (cpuFreq->plugin);
+    cpufreq_warn_reset ();
+  }
+
+  switch (cpuFreq->options->show_cpu)
+  {
+  case CPU_MIN:
     cpu = cpufreq_cpus_calc_min ();
-  else if (cpuFreq->options->show_cpu == CPU_AVG)
+    break;
+  case CPU_AVG:
     cpu = cpufreq_cpus_calc_avg ();
-  else if (cpuFreq->options->show_cpu == CPU_MAX)
+    break;
+  case CPU_MAX:
     cpu = cpufreq_cpus_calc_max ();
+    break;
+  default:
+    if (cpuFreq->options->show_cpu >= 0 && cpuFreq->options->show_cpu < (gint) cpuFreq->cpus->len)
+      cpu = g_ptr_array_index (cpuFreq->cpus, cpuFreq->options->show_cpu);
+  }
 
   return cpu;
 }
@@ -408,6 +470,8 @@ cpufreq_update_plugin (gboolean reset_label_size)
   gboolean ret;
 
   cpu = cpufreq_current_cpu ();
+  if (!cpu)
+      return FALSE;
 
   if (reset_label_size)
   {
@@ -483,7 +547,7 @@ cpufreq_restart_timeout (void)
 #ifdef __linux__
   g_source_remove (cpuFreq->timeoutHandle);
   cpuFreq->timeoutHandle = g_timeout_add_seconds (
-    cpuFreq->options->timeout, (GSourceFunc)cpufreq_update_cpus, NULL);
+    cpuFreq->options->timeout, cpufreq_update_cpus, NULL);
 #endif
 }
 
@@ -549,6 +613,14 @@ cpufreq_prepare_label (CpuFreqPlugin *cpufreq)
 {
   if (cpufreq->label)
   {
+    if (cpuFreq->label_css_provider)
+    {
+      gtk_style_context_remove_provider (
+        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (cpuFreq->label)),
+        GTK_STYLE_PROVIDER (cpuFreq->label_css_provider));
+      cpuFreq->label_css_provider = NULL;
+    }
+
     gtk_widget_destroy (cpufreq->label);
     cpufreq->label = NULL;
   }
@@ -578,7 +650,7 @@ cpufreq_widgets (void)
   provider = gtk_css_provider_new ();
   gtk_css_provider_load_from_data (provider, css, -1, NULL);
   gtk_style_context_add_provider (
-    GTK_STYLE_CONTEXT (gtk_widget_get_style_context (GTK_WIDGET (cpuFreq->button))),
+    GTK_STYLE_CONTEXT (gtk_widget_get_style_context (cpuFreq->button)),
     GTK_STYLE_PROVIDER(provider),
     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   g_free(css);
@@ -627,13 +699,13 @@ cpufreq_read_config (void)
   cpuFreq->options->timeout             = xfce_rc_read_int_entry  (rc, "timeout", 1);
   if (cpuFreq->options->timeout > TIMEOUT_MAX || cpuFreq->options->timeout < TIMEOUT_MIN)
     cpuFreq->options->timeout = TIMEOUT_MIN;
-  cpuFreq->options->show_cpu            = xfce_rc_read_int_entry  (rc, "show_cpu",  0);
-  cpuFreq->options->show_icon           = xfce_rc_read_bool_entry (rc, "show_icon",  TRUE);
+  cpuFreq->options->show_cpu            = xfce_rc_read_int_entry  (rc, "show_cpu", CPU_DEFAULT);
+  cpuFreq->options->show_icon           = xfce_rc_read_bool_entry (rc, "show_icon", TRUE);
   cpuFreq->options->show_label_freq     = xfce_rc_read_bool_entry (rc, "show_label_freq", TRUE);
-  cpuFreq->options->show_label_governor =	xfce_rc_read_bool_entry (rc, "show_label_governor", TRUE);
-  cpuFreq->options->show_warning        =	xfce_rc_read_bool_entry (rc, "show_warning", TRUE);
-  cpuFreq->options->keep_compact        =	xfce_rc_read_bool_entry (rc, "keep_compact", FALSE);
-  cpuFreq->options->one_line            =	xfce_rc_read_bool_entry (rc, "one_line", FALSE);
+  cpuFreq->options->show_label_governor = xfce_rc_read_bool_entry (rc, "show_label_governor", TRUE);
+  cpuFreq->options->show_warning        = xfce_rc_read_bool_entry (rc, "show_warning", TRUE);
+  cpuFreq->options->keep_compact        = xfce_rc_read_bool_entry (rc, "keep_compact", FALSE);
+  cpuFreq->options->one_line            = xfce_rc_read_bool_entry (rc, "one_line", FALSE);
 
   if (!cpuFreq->options->show_label_freq && !cpuFreq->options->show_label_governor)
     cpuFreq->options->show_icon = TRUE;
@@ -671,8 +743,8 @@ cpufreq_write_config (XfcePanelPlugin *plugin)
   rc = xfce_rc_simple_open (file, FALSE);
   g_free(file);
 
-  xfce_rc_write_int_entry	 (rc, "timeout",             cpuFreq->options->timeout);
-  xfce_rc_write_int_entry	 (rc, "show_cpu",            cpuFreq->options->show_cpu);
+  xfce_rc_write_int_entry  (rc, "timeout",             cpuFreq->options->timeout);
+  xfce_rc_write_int_entry  (rc, "show_cpu",            cpuFreq->options->show_cpu);
   xfce_rc_write_bool_entry (rc, "show_icon",           cpuFreq->options->show_icon);
   xfce_rc_write_bool_entry (rc, "show_label_freq",     cpuFreq->options->show_label_freq);
   xfce_rc_write_bool_entry (rc, "show_label_governor", cpuFreq->options->show_label_governor);
@@ -796,7 +868,7 @@ cpufreq_construct (XfcePanelPlugin *plugin)
   cpufreq_widgets ();
 
   cpuFreq->timeoutHandle = g_timeout_add_seconds (
-    cpuFreq->options->timeout, (GSourceFunc) cpufreq_update_cpus, NULL);
+    cpuFreq->options->timeout, cpufreq_update_cpus, NULL);
 #else
   xfce_dialog_show_error (NULL, NULL, _("Your system is not supported yet!"));
 #endif /* __linux__ */
